pub struct __IncompleteArrayField<T>(/* private fields */);Implementations
Sourceimpl<T> __IncompleteArrayField<T>
impl<T> __IncompleteArrayField<T>
Sourceimpl<U> __IncompleteArrayField<U>
impl<U> __IncompleteArrayField<U>
Sourcepub fn as_entry_ptr_mut(ptr: *mut __IncompleteArrayField<U>) -> *mut U
pub fn as_entry_ptr_mut(ptr: *mut __IncompleteArrayField<U>) -> *mut U
Utility for casting __IncompleteArrayField
Sourcepub fn entry_size(_: *const __IncompleteArrayField<U>) -> usize
pub fn entry_size(_: *const __IncompleteArrayField<U>) -> usize
Utility for getting the size of the interior type Note we must use a raw pointer rather than a reference here, because the compiler thinks the field itself may be unaligned due to the struct being packed